#include <SPI.h>      // Bibliotecas necessárias ao uso do RFID
#include <MFRC522.h>
#include <Wire.h>     // Bibliotecas necessárias para utilizar o LCD
#include <LiquidCrystal_I2C.h>
#include <Servo.h> // Biblioteca para o funcionamento do servo
#include <PCF8574.h> // Biblioteca para utilizar o PCF

// Variáveis a utilizar na iluminação da rua
const int luzRua = 2;
const int btLuzRua = 3;
bool luzRuaON = false; // Indica que a luz da rua está desligada

// Variáveia a utilizar nos lugares de estacionamento
const int PHR_1 = A0;
const int PHR_2 = A1;
const int PHR_3 = A2;
const int PHR_4 = A3;
const int valorPHR = 500; // Valor para que haja alteração de estado do LED

PCF8574 pcf(0x20); // Endereço utilizado pelo nosso PCF8574P
const int lugar_1_L = 0; // Led verde
const int lugar_1_O = 1; // Led Vermelho
const int lugar_2_L = 2; // Led verde
const int lugar_2_O = 3; // Led Vermelho
const int lugar_3_L = 5; // Led verde
const int lugar_3_O = 4; // Led Vermelho
const int lugar_4_L = 7; // Led verde
const int lugar_4_O = 6; // Led Vermelho

Servo servoE;
Servo servoS;
const int portServoE  = 6;  // Servo de entrada no parque
const int portServoS  = 7;  // Servo de saída do parque
int servoEInicio      = 90;
int servoEFim         = 0;
int servoSInicio      = 0;
int servoSFim         = 90;
int servoTemp         = 30; // Delay de abertura ou fecho da cancela
bool querSair         = false; // Para termos uma indicação que o veículo vai entrar no parque

LiquidCrystal_I2C lcd(0x27, 16, 2); // O endereço do LCD, em príncipio, é 0x27
 
#define SS_PIN 10
#define RST_PIN 9
MFRC522 mfrc522(SS_PIN, RST_PIN);   // Cria o objeto mfrc522.

void setup() {
  Serial.begin(9600);

  pinMode(luzRua, OUTPUT);
  pinMode(btLuzRua, INPUT);

  pinMode(PHR_1, INPUT);
  pinMode(PHR_2, INPUT);
  pinMode(PHR_3, INPUT);
  pinMode(PHR_4, INPUT);

  //indicar o pinMode de cada uma das portas do PCF
  for(int i=0; i<=7; i++){
    pcf.pinMode(i, OUTPUT);
  }
  pcf.begin();

  servoE.attach(portServoE);  // Identificamos as portas que os servos vão usar
  servoS.attach(portServoS);
  
  lcd.init(); // Inicializa o LCD
  lcd.setBacklight(HIGH); // Liga o LED do LCD

  SPI.begin();          // Inicia  SPI - Serial Peripheral Interface - bus que permite a comunicação entre os vários componentes
  mfrc522.PCD_Init();   // Inicia o leitor de cartões RFID
  Serial.println("Aproxima o cartao do leitor...");
  Serial.println();
  
  // Inicializar sistema - garantimos que todo o sistema está de acordo com o que pretendemos
  digitalWrite(luzRua, LOW); // Desligamos a luz da Rua
  for(int i=0; i<=7; i++){  // Desligamos os LED RGB 
    pcf.digitalWrite(i, LOW);
  }  
  servoE.write(servoEInicio); // Inicializa os Servos na posição 0 - Fechados
  servoS.write(servoSInicio);
  lcd.clear();
  lcd.print("Parque CSG");
}

void loop() {
  // Ligar e desligar a luz da rua
  // Ler valor do botão
  int valueBt = digitalRead(btLuzRua);
  Serial.println(valueBt);
  if(valueBt == 1){
    if(!luzRuaON){ 
      digitalWrite(luzRua, HIGH); // Liga a luz da Rua
      luzRuaON = true; // flag que indica que a luz está ligada
    } else { 
      digitalWrite(luzRua, LOW); // Desliga a luz da Rua
      luzRuaON = false; // flag que indica que a luz está desligada
    }
  }
  delay(300); // Para não ler vários valores seguidos


  // Leitura dos photoresistores do parque
  int valuePHR_1 = analogRead(PHR_1);
  int valuePHR_2 = analogRead(PHR_2);
  int valuePHR_3 = analogRead(PHR_3);
  int valuePHR_4 = analogRead(PHR_4);

  Serial.print(valuePHR_1);
  Serial.print(" - ");
  Serial.print(valuePHR_2);
  Serial.print(" - ");
  Serial.print(valuePHR_3);
  Serial.print(" - ");
  Serial.print(valuePHR_4);
  Serial.print(" - ");

  if(valuePHR_1 < valorPHR){
    pcf.digitalWrite(lugar_1_L, LOW);
    pcf.digitalWrite(lugar_1_O, HIGH);
  } else {    
    pcf.digitalWrite(lugar_1_L, HIGH);
    pcf.digitalWrite(lugar_1_O, LOW);
  }

  if(valuePHR_2 < valorPHR){
    pcf.digitalWrite(lugar_2_L, LOW);
    pcf.digitalWrite(lugar_2_O, HIGH);
  } else {    
    pcf.digitalWrite(lugar_2_L, HIGH);
    pcf.digitalWrite(lugar_2_O, LOW);
  }

  if(valuePHR_3 < valorPHR){
    pcf.digitalWrite(lugar_3_L, LOW);
    pcf.digitalWrite(lugar_3_O, HIGH);
  } else {    
    pcf.digitalWrite(lugar_3_L, HIGH);
    pcf.digitalWrite(lugar_3_O, LOW);
  }

  if(valuePHR_4 < valorPHR){
    pcf.digitalWrite(lugar_4_L, LOW);
    pcf.digitalWrite(lugar_4_O, HIGH);
  } else {    
    pcf.digitalWrite(lugar_4_L, HIGH);
    pcf.digitalWrite(lugar_4_O, LOW);
  }

  // Procura novos cartões
  if (mfrc522.PICC_IsNewCardPresent()){
    if (mfrc522.PICC_ReadCardSerial()){
      //Mostra UID no monitor serial
      Serial.print("UID da tag :");
      String conteudo= "";
      
      for (byte i = 0; i < mfrc522.uid.size; i++){
         Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
         Serial.print(mfrc522.uid.uidByte[i], HEX);
         conteudo.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
         conteudo.concat(String(mfrc522.uid.uidByte[i], HEX));
      }
      Serial.println();
      Serial.print("Mensagem : ");
      conteudo.toUpperCase();
      lcd.clear();
      lcd.setCursor(0, 0);
      if (conteudo.substring(1) == "34 94 25 D9"){ //UID 1 - Chaveiro
        if(!querSair){ // O veículo vai entrar no parque
          Serial.println("Bem-vindo!");
          Serial.println();
          lcd.print("Seja Bem-vindo :D"); // escreve a mensagem        
          for (int i=servoEInicio; i>=servoEFim; i-=2){ // Criar um ciclo para abrir a cancela
            servoE.write(i);
            delay(servoTemp);
          }      
          delay(4000);      
          // Criar um ciclo para fechar a cancela
          for (int i=servoEFim; i<=servoEInicio; i+=2){
            servoE.write(i);
            delay(servoTemp);
          }
          querSair = true;
        } else { // O veículo vai sair do parque
          Serial.println("Obrigado!");
          Serial.println();
          lcd.print("Obrigado!"); // escreve a mensagem   
          lcd.setCursor(0, 1);   
          lcd.print("Volte sempre :D"); 
          for (int i=servoSInicio; i<=servoSFim; i+=2){ // Criar um ciclo para abrir a cancela
            servoS.write(i);
            delay(servoTemp);
          }      
          delay(4000);      
          // Criar um ciclo para fechar a cancela
          for (int i=servoSFim; i>=servoSInicio; i-=2){
            servoS.write(i);
            delay(servoTemp);
          }          
          querSair = false;
        }
      } else {
        Serial.println("Não te conheço, quem és? :D ");
        Serial.println();
        lcd.print("Cartao Invalido :("); // escreve a mensagem        
      }  
      delay(2000);
      lcd.clear();
      lcd.print("Parque CSG");
    }
  }

}
